#!/usr/local/bin/perl
#
#  Split "hdp dumpvd" records be one-per-line.
#
$printrecord = "off";
print "\n";

#
#  Get command line elements.
#
if      ( @ARGV == 0 ) {
   $first = "0";
   $last  = "100000000";
} elsif ( @ARGV == 1 ) {
   $first = $ARGV[0];
   $last  = $first;
} elsif ( @ARGV >= 1 ) {
   $first = $ARGV[0];
   $last  = $ARGV[1];
}

#
#  We take line from STDIN.
#
while( $line = <STDIN> ) {

   chop( $line );
   
   #
   #  If we see start of the next Vdata, we turn record dump off.
   #
   if ( $line =~ /^Vdata\:/ ) {
      
      $printrecord = "off";
      print "\n----------------------------------------------------------------------\n";
      
      #
      #  We also need to re-initialize field name array.
      #
      @fields = ( "record" );
      $reccount = "0";
      
   }
   
   #
   #  If we see record table header, we turn record dump on.
   #
   if ( $line =~ /^Loc\.\s+Data\s*$/ ) {
      $printrecord = "on";
      print "\n";
   }
   
   #
   #  If record dump is on and line is not blank or table header,
   #  we consider current lineto be a record and try to dump it.
   #
   if ( $printrecord eq "on"    && 
        ! ( $line =~ /^\s*$/ )  && 
        ! ( $line =~ /^Loc\.\s+Data\s*$/ ) ) {
   
      #
      #  First, we chop off the ";" at the end of the line.
      #
      $line =~ s/\s+;\s*$//;
      
      #
      #  Then we split line into records, if there is more than
      #  one record in the line, and try to dump each record.
      #
      @records = split( /\s*;\s*/, $line );
		
		#
		#	If there are multiple records, prefix each record with index 
		#	1+ by the record number.
		#
		if ( $#records > 0 ) {
			
			#
			#	Pull out first record index.
			#
			( $idx, $tmp ) = split ( ' ', $records[0] );
			
			#
			#	Prefix all other records.
			#
			foreach $i ( 1 .. $#records ) {
				$idx++;
				$records[$i] = $idx." ".$records[$i];
			}
			
		}
      
      foreach $rec ( @records ) {
      
         #
         #  Check whether current record is with requested boundaries. 
         #
         if ( $reccount >= $first && $reccount <= $last ) {
         
            #
            #  Replace 1+ spaces that follow record index, the fist 
            #  token in the record, with two spaces and split the 
            #  the record at two spaces.
            #
            $rec =~ s/\s+/  /;
                     
            @tokens = split( /  /, $rec );
            
            #
            #  Check if number of tokens match the number of names.
            #
            if ( $#tokens == $#fields ) {
            
               #
               #  We print each of the tokens.
               #
               foreach $i ( 0 .. $#fields ) {
                  printf "%30s %s\n", $fields[$i], $tokens[$i];
               }
               
            } else {
            
               #
               #  Numbers of elements don't match. Complain.
               #
               print "Number of tokens, $#tokens, doesn't match ".
                     "number of fields, $#fields, for the record ".
                     "$reccount.";
               
            }
            
            print "\n";
   
         }
         
         #
         #  Increase record count.
         #
         $reccount++;
         
      }
      
   }
   
   #
   #  If record dump is off, we need to look for Vdata and 
   #  field names.
   #
   if ( $printrecord eq "off" ) {
   
		#
		#	Number of records is on the line contaning 
		#	"number of records =" 
		#
      if ( $line =~ /^\s*number\s+of\s+records\s+=\s+/ ) {
         print "$line\n";
         
      }

      #
      #  Vdata name is always on the line contanig "name = "
      #
      if ( $line =~ /^\s*name\s+=\s+/ ) {
         print "$line\n\n";
         
      }

      #
      #  Field names are on the lines contaning "- field index ".
      #
      if ( $line =~ /^\-\s+field\s+index\s+/ ) {
      
         #
         #  We need to pull out field name and save it for future
         #  values tagging.
         #
         ($tmp,$name) = split( /\[/, $line );
         ($name,$tmp) = split( /\]/, $name );
         push( @fields, $name );
         print "$line\n";
      }
      
   }
   
}

print "\n----------------------------------------------------------------------\n";

exit;
