#!/usr/local/bin/perl
#
#  Split "msop_query" records be one-per-line.
#
$reccount = "0";
$printrecord = "off";
print "\n";

#
#  Get command line elements.
#
if      ( @ARGV == 0 ) {
   $first = "0";
   $last  = "100000000";
} elsif ( @ARGV == 1 ) {
   $first = $ARGV[0];
   $last  = $first;
} elsif ( @ARGV >= 1 ) {
   $first = $ARGV[0];
   $last  = $ARGV[1];
}

#
#  We take line from STDIN.
#
while( $line = <STDIN> ) {

   chop( $line );

   #
   #  If record dump is on, we consider current line to be a record 
   #  and try to dump it.
   #
   if ( $printrecord eq "on" ) {
         
      #
      #  Check whether current record is with requested boundaries. 
      #
      if ( $reccount >= $first && $reccount <= $last ) {
      
         #
         #  Split the the record at spaces.
         #
         @tokens = split( ' ', $line );
         
         #
         #  Check if number of tokens match the number of names.
         #
         if ( $#tokens == $#fields ) {
         
            printf "%30s %s   %s %s\n", 
                   "record", $reccount, $fields[0], $tokens[0];

            #
            #  We print each of the tokens.
            #
            foreach $i ( 1 .. $#fields ) {
               printf "%30s %s\n", $fields[$i], $tokens[$i];
            }
            
         } else {
         
            #
            #  Numbers of elements don't match. Complain.
            #
            print "Number of tokens, $#tokens, doesn't match ".
                  "number of fields, $#fields, for the record ".
                  "$reccount.";
            
         }
         
         print "\n";

      }
      
      #
      #  Increase record count.
      #
      $reccount++;
      
   }
   
   #
   #  If record dump is off, we need to look for field names.
   #
   if ( $printrecord eq "off" ) {
   
      #
      #  Channel IDs are on the line lines contaning "SCLK 25".
      #  Channel names are on a line two lines below it.
      #
      if ( $line =~ /^            SCLK 25 / ) {
      
         #
         #  We need to pull out channels IDs and save them for future
         #  values tagging.
         #
         $line =~ s/            SCLK 25 /SCLK /;
         @fieldids = split( ' ', $line );

         #
         #  Skip two lines.
         #
         $line = <STDIN>;
         $line = <STDIN>;
         $line = <STDIN>;
         chop( $line );
         
         #
         #  We also add channel names to IDs to make tagging more 
         #  complete.
         #
         @fields = split( ' ', $line );
         @fields = ( " ", @fields );
      
         foreach $i ( 0 .. $#fields ) {
            $fields[$i] = "$fields[$i] ($fieldids[$i])";
         }
         
         #
         #  Set process record flag to on.
         #  
         $printrecord = "on";

         print "\n----------------------------------------------------------------------\n";
         print "\n"; 

      }
      
   }
   
}

print "\n----------------------------------------------------------------------\n";

exit;
