
/*    
     
   GNS EDR Structure definitions -- GEM Manual/Calibration Mode Data   
  
   BVS/NAIF, June 7, 2000
  
      Changed sclk_subseconds type from uint8 to uint16 to preserve 
      full resolution of the fractional part of SCLK. Updated 
      description to reflect this change. Added APID_status bit 
      breakdown description.
     
   BVS/NAIF, March 2, 2000
  
      Reformatted comments; moved dimension constants to 
      gnd_edr_const.h; included gnd_edr_const.h.
  
   BVS/NAIF, February 17, 2000  
     
*/  

#include "hdfi.h"
#include "gns_edr_const.h"

/*  
   
   GEM Manual/Calibration data structure -- APID # 33
   
      sclk_seconds      -- start SCLK seconds
      
      sclk_subseconds   -- start fractions (1/65536th) of SCLK second
      
      APID_status       -- APID status word (? bit order):

                              Bits 0-7 = Azimuth_Sampled_Indication
                              Bit 8 = St_man_cal_fl
                              Bit 9 = Checksum_Error_Occur_Indication
                              Bit 10 = Status_Early_Sync
                              Bit 11 = Status_Synthetic_Sync
                              Bit 12 = Status_Test_Data
                              Bits 13-15 = St_cal_spin
      
      step_time         -- step time that specified in the command,
                           units of 0.1 microseconds
                           
      integ_time        -- detector integration time, 10 microsecond 
                           units
                           
      settle_time       -- power supply settling time, 10 microsecond 
                           units
                           
      counts[7][5]      -- CEM counts for each CEM/level
   
*/

struct GEM_ManCal {

   uint32   sclk_seconds;
   uint16   sclk_subseconds;
   uint16   APID_status;
   uint16   step_time;
   uint16   integ_time;
   uint16   settle_time;
   uint8    counts[GEM_CEMS][GEM_SPIN_LEVELS];
                                       
                                       
};


