
/*  
   
   GNS EDR Structure definitions -- GEM Normal Mode Data 
   
   BVS/NAIF, June 7, 2000
  
      Changed sclk_subseconds type from uint8 to uint16 to preserve 
      full resolution of the fractional part of SCLK. Updated 
      description to reflect this change. Added APID_status bit
      breakdown description.
        
   BVS/NAIF, March 2, 2000
  
      Reformatted comments; moved dimension constants to 
      gnd_edr_const.h; included gnd_edr_const.h.
  
   BVS/NAIF, February 17, 2000
   
*/

#include "hdfi.h"
#include "gns_edr_const.h"


/*  
   
   GEM Normal data structure -- APID # 31
   
      APID_status          -- APID status word (? bit order):
      
                                 bit 4-9 = Status_Checksum_Error
                                 bit 0 = Status_Early_Sync
                                 bit 1 = Status_Synthetic_Sync
                                 bit 2 = Status_Test_Data
                                 bit 3 = Status_Use_Data
                                 
      sclk_seconds[4]      -- 1-4 spin start SCLK seconds
      
      sclk_subseconds[4]   -- 1-4 spin start fractions (1/65536th) of 
                              SCLK second
                              
      step_time[4]         -- 1-4 spin step time that specified in 
                              the command, units of 0.1 microseconds
                              
      level[4]             -- 1-4 spin ADC values from ESA voltage 
                              readback for the voltage level, DAC 
                              units (0-4095d)
                              
      step_size[4]         -- voltage level step size that was 
                              specified in command, units of 0.01 
                              percent
                              
      integ_time           -- detector integration time, 10 microsecond
                              units
                              
      settle_time          -- power supply settling time, 10 microsecond
                              units
                              
      ESA_level[20]        -- ESA levels, (DAC units (0-4095d)?)
      
      counts[24][7][20]    -- CEM counts for each sector/CEM/level
   
*/

struct GEM_Normal {

   uint16   APID_status;
   uint32   sclk_seconds[GEM_SPINS];
   uint16   sclk_subseconds[GEM_SPINS];
   uint16   step_time[GEM_SPINS];
   uint16   level[GEM_SPINS];
   uint16   step_size[GEM_SPINS];
   uint16   integ_time;
   uint16   settle_time;
   uint16   ESA_level[GEM_CYCLE_LEVELS];
   uint8    counts[GEM_AZIMUTH_SECTORS][GEM_CEMS][GEM_CYCLE_LEVELS];
                                       
};


