
/*  
   
   GNS EDR Structure definitions -- GEM Raw Count Data 
   
   BVS/NAIF, November 11, 2000
   
*/

#include "hdfi.h"
#include "gns_edr_const.h"


/*  
   
   GEM raw count structure -- APID # 41
   
      sclk_seconds         -- spin start SCLK seconds
      
      sclk_subseconds      -- spin start fractions (1/65536th) of 
                              SCLK second
                              
      start_req[24]        -- voltage requested start level that 
                              was specified in acquire or manual 
                              command, DAC units (0-4095d)
                              
      step_size[24]        -- voltage level step size that was 
                              specified in command, units of 00.001 
                              percent, 00 if manual acquisition
                              
      step_time[24]        -- step time that was specified in the 
                              command, units of 0.1 microseconds.
                              
      angle[24]            -- sequence number implying angular 
                              position of data set (0-23)
                              
      ESA_set[24][5]       -- ADC value from ESA voltage DAC 
                              readback for each sector/voltage 
                              level (0-4095d)
      
      ESA_level[24][5]     -- ADC value from ESA voltage readback 
                              for each sector/voltage level (0-4095d)
      
      counts[24][5][7]     -- CEM counts for each sector/voltage
                              level/CEM, (0-65535)
   
*/

struct GEM_Raw {

   uint32   sclk_seconds;
   uint16   sclk_subseconds;
   uint16   start_req[GEM_AZIMUTH_SECTORS];
   uint16   step_size[GEM_AZIMUTH_SECTORS];         
   uint16   step_time[GEM_AZIMUTH_SECTORS];          
   uint16   angle[GEM_AZIMUTH_SECTORS];              
   uint16   ESA_set[GEM_AZIMUTH_SECTORS][GEM_SPIN_LEVELS];
   uint16   ESA_level[GEM_AZIMUTH_SECTORS][GEM_SPIN_LEVELS];
   uint16   counts[GEM_AZIMUTH_SECTORS][GEM_SPIN_LEVELS][GEM_CEMS];
                                       
};
