
/*
  
   GNS EDR Structure definitions -- GIM Housekeeping Data 
  
   BVS/NAIF, August 23, 2000
  
      Per information provided by Eric Dors on 08/21/00 added 
      description of bits 10-15 of CMD_status.
        
   BVS/NAIF, June 7, 2000
  
      Changed sclk_subseconds type from uint8 to uint16 to preserve 
      full resolution of the fractional part of SCLK. Updated 
      description to reflect this change. Added APID_status, SW_status
      and CMD_status bit breakdown description.
        
   BVS/NAIF, March 2, 2000
  
      Reformatted comments; moved dimension constants to 
      gnd_edr_const.h; included gnd_edr_const.h.
  
   BVS/NAIF, February 17, 2000
  
*/

#include "hdfi.h"
#include "gns_edr_const.h"

/*
   
   GIM Housekeeping data structure -- APID # 29
   
      sclk_seconds      -- SCLK seconds 

      sclk_subseconds   -- fractions (1/65536th) of SCLK second 

      APID_status       -- APID status word (? bit order): 

                              bits 0-7 = Azimuth_Sampled_Indication
                              bits 8-9 = Sample_Mode
                              bit 10 = Checksum_Error_Occur_Indication
                              
      start_level       -- voltage requested start level that was 
                           specified in acquire or manual command, 
                           DAC units (0-4095d) 

      step_size         -- voltage level step size that was specified 
                           in command, units of 0.01 percent, 0 if 
                           manual acqusition                                     

      set[10]           -- ADC values from ESA voltage DAC readback 
                           for the voltage levels 0-9, DAC units 
                           (0-4095d)              

      level[10]         -- ADC values from ESA voltage readback for 
                           the voltage levels 0-9, DAC units (0-4095d)              

      CEM_set           -- ADC value from CEM voltage DAC readback, 
                           (0-4095d)              

      CEM_level         -- ADC value from CEM voltage readback, 
                           (0-4095d)              

      AMP_set           -- ADC value from AMP voltage DAC readback, 
                           (0-4095d)              

      MON_5V            -- ADC value from 5 volt monitor, (0-4095d)                        

      MON_12V           -- ADC value from 12 volt monitor, (0-4095d)                        

      SW_status         -- GIM SW status word (? bit order):              

                              bit 0 = HV_ENABLE
                              bit 1 = HV_LIMIT
                              bit 2 = RAM_TEST
                              bit 3 = ROM_TEST
                              bit 4 = INTERUPT_TEST
                              bit 5 = TIMING_TEST
                              bit 6 = DAC1_TEST
                              bit 7 = DAC2_TEST
                              bit 8 = DAC3_TEST
                              bit 9 = PROCESSOR_RESET
                              bits 10-11 = TEST_PULSER
                              bit 14 = LAST_SYNC_EARLY_FLAG
                              bit 15 = SIMULATOR_DATA
                           
      CMD_status        -- GIM CMD status word (? bit order):              

                              bits 0-7 = Command_Status_Counter
                              bit 8 = CMD_Status_Bad_Command_Flag
                              bit 9 = CMD_Status_Bad_Parameter_Flag
                              bits 10-14 = reserved, 0
                              bit 15 = AMP level stepping (1= AMP 
                                       stepping/SET, AMP_SET swapping 
                                       in effect)
   
*/

struct GIM_Housekeeping {

   uint32   sclk_seconds;
   uint16   sclk_subseconds;
   uint16   APID_status;
   uint16   start_level;
   uint16   step_size;
   uint16   set[GIM_SPIN_LEVELS];
   uint16   level[GIM_SPIN_LEVELS];
   uint16   CEM_set;
   uint16   CEM_level;
   uint16   AMP_set;
   uint16   MON_5V;
   uint16   MON_12V;
   uint16   SW_status;
   uint16   CMD_status;
                                       
};


