
/*  
   
   GNS EDR Structure definitions -- GIM Raw Count Data 
   
   BVS/NAIF, November 11, 2000
   
*/

#include "hdfi.h"
#include "gns_edr_const.h"


/*  
   
   GIM raw count structure -- APID # 42
   
      sclk_seconds         -- spin start SCLK seconds
      
      sclk_subseconds      -- spin start fractions (1/65536th) of 
                              SCLK second
                              
      start_req[40]        -- voltage requested start level that 
                              was specified in acquire or manual 
                              command, DAC units (0-4095d)
                              
      step_size[40]        -- voltage level step size that was 
                              specified in command, units of 00.001 
                              percent, 00 if manual acquisition
                              
      step_time[40]        -- step time that was specified in the 
                              command, units of 0.1 microseconds.
                              
      angle[40]            -- sequence number implying angular 
                              position of data set (0-23)
                              
      ESA_set[40][10]      -- ADC value from ESA voltage DAC 
                              readback for each sector/voltage 
                              level (0-4095d)
      
      ESA_level[40][10]    -- ADC value from ESA voltage readback 
                              for each sector/voltage level (0-4095d)
      
      counts[40][10][8]    -- CEM counts for each sector/voltage
                              level/CEM, (0-65535)
   
*/

struct GIM_Raw {

   uint32   sclk_seconds;
   uint16   sclk_subseconds;
   uint16   start_req[GIM_AZIMUTH_SECTORS];
   uint16   step_size[GIM_AZIMUTH_SECTORS];         
   uint16   step_time[GIM_AZIMUTH_SECTORS];          
   uint16   angle[GIM_AZIMUTH_SECTORS];              
   uint16   ESA_set[GIM_AZIMUTH_SECTORS][GIM_SPIN_LEVELS];
   uint16   ESA_level[GIM_AZIMUTH_SECTORS][GIM_SPIN_LEVELS];
   uint16   counts[GIM_AZIMUTH_SECTORS][GIM_SPIN_LEVELS][GIM_CEMS];
                                       
};
